use indexmap::IndexSet;
use snafu::Snafu;
use std::result;
use {DataType, DataValueRaw};

pub type Result<T> = result::Result<T, Error>;

#[derive(Clone, Eq, PartialEq, Debug, Snafu)]
pub enum Error {
    #[snafu(display(
        "value {:?} is not in range for type {:?}",
        value,
        datatype
    ))]
    ValueOutOfRangeForType {
        value: DataValueRaw,
        datatype: DataType,
    },
    #[snafu(display(
        "expected datatype compatible with {:?} found {:?}",
        expected,
        found
    ))]
    IncompatibleDataType { expected: DataType, found: DataType },
    #[snafu(display(
        "parameter {:?} with index {} of mask range 64",
        parameter,
        index
    ))]
    ParameterOutOfMaskRange { parameter: String, index: usize },
    #[snafu(display(
        "mask {:#b} has bit {} for non-existing parameters set",
        mask,
        index
    ))]
    ParameterMaskOutOfRange { mask: u64, index: usize },
    #[snafu(display(
        "parameters {:?} are not in parameter set",
        parameters
    ))]
    ParametersNotInParametrSet { parameters: IndexSet<String> },
}
